# ocspkg - a Python library for RPM Packagers
#
# Copyright (C) 2011 Red Hat Inc.
# Author(s): Jesse Keating <jkeating@redhat.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
# the full text of the license.

import pyrpkg
import os
import git
import re
import six

from datetime import datetime, timedelta

# doc/ocspkg_man_page.py uses the 'cli' import
from . import cli  # noqa
from .lookaside import FedoraLookasideCache
from pyrpkg.utils import cached_property
from pkg_resources import get_distribution, parse_version

try:
    from distro import linux_distribution  # noqa
except ImportError:
    from platform import linux_distribution  # noqa


# ocstech: not support bodhi


class Commands(pyrpkg.Commands):

    def __init__(self, *args, **kwargs):
        """Init the object and some configuration details."""

        super(Commands, self).__init__(*args, **kwargs)

        self.source_entry_type = 'bsd'
        self.distgit_namespaced = False #ocstech：git clone repo, disabled ns(namespace/repo)
        # un-block retirement of packages (module retirement is allowed by default)
        if 'rpms' in self.block_retire_ns:
            self.block_retire_ns.remove('rpms')

    def load_user(self):
        """This sets the user attribute, based on the Fedora SSL cert."""
        fedora_upn = os.path.expanduser('~/.fedora.upn')
        if os.path.exists(fedora_upn):
            with open(fedora_upn, 'r') as f:
                self._user = f.read().strip()
        else:
            self.log.debug('Could not get user from .fedora.upn, falling back'
                           ' to default method')
            super(Commands, self).load_user()

    @cached_property
    def lookasidecache(self):
        """A helper to interact with the lookaside cache

        We override this because we need a different download path.
        """
        return FedoraLookasideCache(
            self.lookasidehash, self.lookaside, self.lookaside_cgi)

    # Overloaded property loaders
    def load_rpmdefines(self):
        """Populate rpmdefines based on branch data"""

        # Determine runtime environment
        self._runtime_disttag = self._determine_runtime_env()

        extra_rpmdefines = []

        # We only match the top level branch name exactly.
        # Anything else is too dangerous and --dist should be used
        # This regex works until after Fedora 99.
        # print ("Branch_merge: ",self.branch_merge) #ocstech: test
        if re.match(r'f\d\d$', self.branch_merge):
            self._distval = self.branch_merge.split('f')[1]
            self._distvar = 'fedora'
            self._disttag = 'fc%s' % self._distval
            self.mockconfig = 'fedora-%s-%s' % (self._distval, self.localarch)
            self.override = 'f%s-override' % self._distval
            self._distunset = 'rhel'
        # Works until RHEL 10
        elif re.match(r'el\d$', self.branch_merge) or \
                re.match(r'epel\d$', self.branch_merge):
            self._distval = self.branch_merge.split('el')[1]
            self._distvar = 'rhel'
            self._disttag = 'el%s' % self._distval
            self.mockconfig = 'epel-%s-%s' % (self._distval, self.localarch)
            self.override = 'epel%s-override' % self._distval
            self._distunset = 'fedora'
        elif re.match(r'epel\d+-playground$', self.branch_merge):
            self._distval = re.search(r'\d+', self.branch_merge).group(0)
            self._distvar = 'rhel'
            self._disttag = 'epel%s.playground' % self._distval
            self.mockconfig = 'epel-%s-%s' % (self._distval, self.localarch)
            self.override = 'epel%s-override' % self._distval
            self._distunset = 'fedora'
        elif re.match(r'epel\d+-next$', self.branch_merge):
            self._distval = re.search(r'\d+', self.branch_merge).group(0)
            self._distvar = 'rhel'
            self._disttag = 'el%s.next' % self._distval
            self.mockconfig = 'epel-next-%s-%s' % (self._distval, self.localarch)
            self.override = 'epel%s-next-override' % self._distval
            self._distunset = 'fedora'
        elif re.match(r'olpc\d$', self.branch_merge):
            self._distval = self.branch_merge.split('olpc')[1]
            self._distvar = 'olpc'
            self._disttag = 'olpc%s' % self._distval
            self.override = 'dist-olpc%s-override' % self._distval
            self._distunset = 'rhel'
        elif re.match(r'eln$', self.branch_merge):
            macros = self._get_build_macros('eln-candidate')
            self._distval = macros['eln']
            self._distvar = 'eln'
            self._disttag = 'eln%s' % macros['eln']
            self.mockconfig = 'fedora-eln-%s' % self.localarch
            self.override = 'eln-override'
            self._distunset = 'fedora'
            extra_rpmdefines = [
                "--define", "el%s 1" % macros['rhel'],
                "--define", "rhel %s" % macros['rhel'],
            ]
        # rawhide (previously master branch)
        elif re.match(r'(rawhide|main)$', self.branch_merge):
            self._distval = self._findrawhidebranch()
            self._distvar = 'fedora'
            self._disttag = 'fc%s' % self._distval
            self.mockconfig = 'fedora-rawhide-%s' % self.localarch
            self.override = None
            self._distunset = 'rhel'
        #ocstech: add ocs,master
        elif re.match(r'(ocs\d\d|master)$', self.branch_merge):
            #self._distval = self._findrawhidebranch() #ocstech: it will establish kojisession when replace koji
            self._distvar = 'opencloudosstream'
            self._disttag = 'ocs%s' % self._distval
            self.mockconfig = 'opencloudosstream-%s' % self.localarch
            self.override = None
            self._distunset = 'ocs'
        # If we don't match one of the above, punt
        else:
            raise pyrpkg.rpkgError('Could not find the release/dist from branch name '
                                   '%s\nPlease specify with --release' %
                                   self.branch_merge)
        self._rpmdefines = ["--define", "_sourcedir %s" % self.layout.sourcedir,
                            "--define", "_specdir %s" % self.layout.specdir,
                            "--define", "_builddir %s" % self.layout.builddir,
                            "--define", "_srcrpmdir %s" % self.layout.srcrpmdir,
                            "--define", "_rpmdir %s" % self.layout.rpmdir,
                            "--define", "_rpmfilename %s" % self.layout.rpmfilename,
                            "--define", "dist %%{?distprefix}.%s" % self._disttag,
                            "--define", "%s %s" % (self._distvar, self._distval),
                            "--eval", "%%undefine %s" % self._distunset,
                            "--define", "%s 1" % self._disttag.replace(".", "_")]
        # TODO: consider removing macro "%s 1; it has unknown/dubious functionality"

        if self._runtime_disttag:
            if self._disttag != self._runtime_disttag:
                # This means that the runtime is known, and is different from
                # the target, so we need to unset the _runtime_disttag
                self._rpmdefines.extend(["--eval", "%%undefine %s" %
                                        self._runtime_disttag])

        if extra_rpmdefines:
            self._rpmdefines.extend(extra_rpmdefines)

    def build_target(self, release):
        if release in ('rawhide', 'main', 'master'): #ocstech: modi custom build_target
            return 'dist-ocs23'
        # if release in ('rawhide', 'main'):
        #     return 'rawhide'
        else:
            return '%s-candidate' % release

    def load_container_build_target(self):
        if self.branch_merge in ('rawhide', 'main'):
            self._container_build_target = 'rawhide-%s-candidate' % self.ns
        else:
            super(Commands, self).load_container_build_target()

    def _tag2version(self, dest_tag):
        """ get the '26' part of 'f26-foo' string """
        return dest_tag.split('-')[0].replace('f', '')

    def _get_build_macros(self, build_target):
        """Get the koji rpm.build.macros for the given build target"""

        if self._kojisession:
            session = self.kojisession
        else:
            session = self.anon_kojisession

        try:
            target = session.getBuildTarget(build_target)
        except Exception as e:
            self.log.error(str(e))
            raise pyrpkg.rpkgError(
                'Cannot access Koji to get build information for {}'.format(build_target))

        buildcfg = session.getBuildConfig(target['build_tag_name'])

        macros = {}
        for extra in buildcfg['extra']:
            if extra.startswith('rpm.macro.'):
                macroname = extra[len('rpm.macro.'):]
                macros[macroname] = buildcfg['extra'][extra]

        return macros

    def _findrawhidebranch(self):
        """Find the right "fedora" for rawhide (previously master branch)"""

        # If we already have a koji session, just get data from the source
        if self._kojisession:
            rawhidetarget = self.kojisession.getBuildTarget('rawhide')
            return self._tag2version(rawhidetarget['dest_tag_name'])

        # Try connect Koji once more, this time with anonymous session.
        try:
            rawhidetarget = self.anon_kojisession.getBuildTarget('rawhide')
        except Exception:
            # We couldn't hit Koji. Continue, because ocspkg may work offline.
            self.log.debug('Unable to query Koji to find rawhide target. Continue offline.')
        else:
            return self._tag2version(rawhidetarget['dest_tag_name'])

        # Create a list of "fedoras"
        fedoras = []

        # Create a regex to find branches that exactly match f##.  Should not
        # catch branches such as f14-foobar
        branchre = r'(ocs\d\d|master)$' # ocstech: modi ocs-branch
        # branchre = r'f\d\d$' # fedora-branch

        # Find the repo refs
        for ref in self.repo.refs:
            # Only find the remote refs
            if type(ref) == git.RemoteReference:
                # Search for branch name by splitting off the remote
                # part of the ref name and returning the rest.  This may
                # fail if somebody names a remote with / in the name...
                if re.match(branchre, ref.name.split('/', 1)[1]):
                    # Add just the simple f## part to the list
                    fedoras.append(ref.name.split('/')[1])
        # print (fedoras,type(fedoras)) # ocstech:
        if fedoras:
            # Sort the list
            fedoras.sort()
            # Start with the last item, strip the f, add 1, return it.
            return int(fedoras[-1].strip('f')) + 1
        else:
            raise pyrpkg.rpkgError('Unable to find rawhide target')

    def _getcommitmessage(self):
        """Returns the latest commit message on the current branch"""

        try:
            commitmsg = six.next(self.repo.iter_commits()).message
        except TypeError:
            commitmsg = ""

        return commitmsg

    def _isrpmautospec(self):
        """Returns whether the spec file uses rpmautospec features"""

        return self._uses_rpmautospec

    def _determine_runtime_env(self):
        """Need to know what the runtime env is, so we can unset anything
           conflicting
        """
        try:
            runtime_os, runtime_version, _ = linux_distribution()
        except Exception:
            return None

        if runtime_os in ['redhat', 'centos']:
            return 'el%s' % runtime_version
        if runtime_os == 'Fedora':
            return 'fc%s' % runtime_version
        if (runtime_os == 'Red Hat Enterprise Linux Server' or
                runtime_os.startswith('CentOS')):
            return 'el{0}'.format(runtime_version.split('.')[0])

    def check_inheritance(self, build_target, dest_tag):
        """Disable check inheritance

        Tag inheritance check is not required in Fedora when make chain build
        in Koji.
        """

    def construct_build_url(self, *args, **kwargs):
        """Override build URL for Fedora Koji build

        In Fedora Koji, anonymous URL should have prefix "git+https://"
        """
        url = super(Commands, self).construct_build_url(*args, **kwargs)
        return 'git+{0}'.format(url)

    def update(self, bodhi_config, template='bodhi.template', bugs=[]):
        """Submit an update to bodhi using the provided template."""
        # ocstech: not support bodhi to update
        pass

    def create_buildroot_override(self, bodhi_config, build, duration,
                                  notes=''):
        # ocstech: not support bodhi to update
        pass

    def extend_buildroot_override(self, bodhi_config, build, duration):
        # ocstech: not support bodhi to update
        pass

    @staticmethod
    def use_local_mock_config(_, force_local):
        if force_local is not None:
            return force_local
        return True


if __name__ == "__main__":
    from ocspkg.__main__ import main
    main()
